// ConsoleApplicationDemoUsage.cpp: Hauptprojektdatei.

#include "PCBI_Lib.h"
#include <iostream>
#include <ctime>
#include <sstream>

using namespace std;

//this is the example class for using events
class MyClass : public Unmanaged
{
public:
	 MyClass() : Unmanaged(){}

	 void OnSelectionChanged()
	 {
		cout << "Event was called"<<endl;
	 }
	 void OnDataLoaded()
	 {
		cout << "new job opend!"<<endl;
	 }
	 void OnActiveLayerChanged()
	 {
		cout << "layer is activated!"<<endl;
	 }
};

int main()
{
#pragma region first demo (use PCB-Investigator from console with event)
	/*-	Load a PCB file*/
	cout << "Load a PCB file " << endl;
	LoadData("D:\\Daten\\Jobs\\Zips\\fpga_asic_64odb.zip");
	
	/*
-	Get notified when a net (or a track, via), a soldering pad or component is selected by the user
*/
	cout << "Create event handler " << endl;

	MyClass* eventReceiver = new MyClass();
	if(RegisterEvent(((Unmanaged*)eventReceiver)))
	{
		cout << "event registrated " << endl;
	}
	else
	{
		cout << "error no event registrated! " << endl;
	}

	/*	
-	Start/Stop/Show/Hide application window (either pre-starts it in the background with no GUI in order to make the first call quicker)
*/
	cout << "Show PCBI window " << endl;
	ShowPCBIWindow();
	
	/*
-	Highlight (blink) a Net, a pin, or a component given its name (supposing that the name is the same as in the netlist) or a list of
*/
	cout << "Call Highlight component " << endl;
	HighlightCMP("Reference");
	/*
-	Zoom to the selected objects
*/
	cout << "Call zoom to component " << endl;
	ZoomToCMP("Reference");
/*
-	Clear highlighted
*/
	cout << "clear selection " << endl;
	ClearAllSelections();
	
	/*
-	An automatic display of the right layer according to the object to be highlighted
*/
	cout << "activate layer for all selected objects " << endl;
	ActivateLayersForSelectedObjects();
	/*
-	A method to get informations (including layer) of the selected objects. 
*/
	cout << "Get component info " << endl;
	char cmpArray[1024];

	GetCMPInfo("Reference",cmpArray, 1024);
	char retArray[1024];
	GetErrorLog(retArray, 1024);

	cout << "Error-LOG: " << retArray << endl;

		std::string var1;
		cout<<"Insert Text and Press Enter to end program"<< endl;
		cin >> var1;
	return 0;
	
#pragma endregion
#pragma region second demo (create one image)
	/*
	char nameArray[128];
	LoadGerberLayerAndReturnLayerName("D:\\Tests\\GerberTests\\RotationArrowsGerber2.ger", nameArray, 128);
	RotateAndMirrorStep(-90,false , false); 

	//Oversize(nameArray, +10); //value in percent, this can be changed in unit depending values if you want (negative = shrinking, positive = growing)

	CreateImage("D:\\Daten\\Jobs\\Gerber_Files\\274x_all\\top.bmp", nameArray, 14111);*/
#pragma endregion

#pragma region third demo (AOI Image creation for gerberfiles with some rotation and mirror settings)
	char nameArray[128];
	LoadGerberLayerAndReturnLayerName("D:\\Tests\\GerberTests\\Panel80x80cm.ger", nameArray, 128);
	//you can use the name to call the CreateImage method with specific layername.

	//this works only before the step changes, because it uses only objects on this layer and with creation of new step it add an empty layer with "image" of the old layer
	
	int retValue = 0;
	for (int i = 0; i < 360; i += 90)
	{
		for (int j = 0; j < 4; j++)
		{
			bool mirrorX = j % 2 == 0 ? false : true;
			bool mirrorY = j >= 2 ? true : false;
			RotateAndMirrorStep(-i,mirrorX , mirrorY); //!after this call the layerbounds are not possible to ask because the internal structure will be changed and no elments on the layers!

			//you can modify the output area by change bounds.
			RECTD bounds;
			bool ok = GetLayerBounds(nameArray, &bounds); //after RotateAndMirrorStep this method gives always 0,0,0,0 back!
			if (!ok || (bounds.left == 0 && bounds.right == 0))
				ok = GetStepBounds(&bounds);	//be carefull after adding more than one layer, if you do NOT use the RotateAndMirrorStep method the step bounds are a combination of all layers (gerberfiles)

			int matrixSize = 4;					//see comment below "AOIMatrixSize" 
			int matrixSizeIndividual = 9000;	//only working if matrixSize = Individual
			int ThreadCount = 16;				//if this is -1 it takes processor count of you machine
			int DPI = 14111;					//28222

			time_t start, ende;
			time(&start);
			std::stringstream ss;
			ss << "D:\\Tests\\GerberTests\\" << i <<"_"<< (mirrorX?"X":"0") << (mirrorY ? "Y" : "0") << "test.bmp";
			cout << "begin with " << ss.str() << "\n";
			//last two parameters for splitting image parts and coutn Threads
			retValue = CreateImage3(ss.str(), nameArray, DPI, false, matrixSize, bounds.left, bounds.right, bounds.bottom, bounds.top, matrixSizeIndividual, ThreadCount);

			time(&ende);
			start -= ende;

			cout << "takes " << -start << " seconds...\n";
			//if (retValue < 0)
			{
				char retArrayErrors[10000];
				GetErrorLog(retArrayErrors, 10000);

				cout << "Error-LOG: " << retArrayErrors << endl;

				ClearLog();
			}
		}

		//remove this for more tests
		break;
	}
	
#pragma endregion

	Save();
	//char retArray[1024];
	GetErrorLog(retArray, 1024);

	cout << "Error-LOG: " << retArray << endl;
	DisposeAllResources();

	return 0;
}


/*public enum AOIMatrixSize
	{
		Low,
		Middle,
		High,
		Special,
		Individual
	}*/
