#include <stdlib.h>
#include <string>
#include <windows.h>
#include "PCBI_Lib.h"

extern "C" {

}


#pragma once

using namespace System;
using namespace System::Collections::Generic;
using namespace PCBI::Automation;
using namespace PCBI;



namespace PCBI_Lib {

	public enum PREPROCESS
	{
		PREPROCESS_ERROR = -1,
		PREPROCESS_SUCCESS = 0,
		PREPROCESS_INVALIDINPUT,
		PREPROCESS_CONVERSIONFAIL,
		PREPROCESS_PIXELCALCFAIL,
		PREPROCESS_FIDUCIALFAIL,
		PREPROCESS_PANELDATAFAIL,
		PREPROCESS_SAVEPREVIEWFAIL,
		PREPROCESS_STRIPESPLITFAIL,
		PREPROCESS_SAVETOFILEFAIL,
		PREPROCESS_RESULTFOLDERFAIL,
		PREPROCESS_LOGGING_FAILURE,
		PREPROCESS_LICENSEERROR,
		PREPROCESS_INITERROR,
		PREPROCESS_NODATA
	};

	public ref class PCBIUnmanaged
	{
	private:
		static	IPCBIWindow^ mainWindow = nullptr;  //main window to work with PCB-Investigator
		static String^ FirstStepName = "";			//for added new steps it is somethimes interesting to know the original step
		static Unmanaged* eventHandlingClassMain;   //event handling by helper class

	public:

		static	String^ GetErrorLog()
		{
			//all errors and warnings in PCBI
			String^ s = IAutomation::GetErrorLog();

			return s;
		}

		static String^ LoadGerberLayer(String^ FullPath)
		{
			//add a single layer to PCBI by loading a gerber file
			if (mainWindow == nullptr) mainWindow = IAutomation::CreateNewPCBIWindow(false, true);
			if (mainWindow == nullptr) return "";
			if (!mainWindow->JobIsLoaded)
			{
				//use always the same directory -> temp data will be removed by overwriting
				String^ jobPath = mainWindow->CreateNewJobDirectory(IAutomation::GetPCBIWorkingDirectory() + "Gerberdata\\ImageCreationProject");
				mainWindow->LoadODBJob(jobPath);
				IAutomation::AddToErrorLog("path of new gerberjob "+ jobPath, false);
			}
			IStep^ step = mainWindow->GetCurrentStep();
			if (step == nullptr) { return ""; IAutomation::AddToErrorLog("data is not loaded!", true);
			}
			FirstStepName = step->Name; //gerber
			return step->AddGerberLayer(FullPath, true);
		}

		static	PREPROCESS LoadData(String^ FullPath)
		{
			//to show PCBI change parameter to true,true
			if (mainWindow == nullptr) mainWindow = IAutomation::CreateNewPCBIWindow(false, true);

			if (mainWindow == nullptr) { IAutomation::AddToErrorLog("Check license, no IPCBIWindow created!", true); return PREPROCESS::PREPROCESS_LICENSEERROR; }

			IPCBIWindow::LoadInformation outInfo;

			bool retValue = mainWindow->LoadData(FullPath, outInfo);

			if (outInfo != IPCBIWindow::LoadInformation::Successful)
			{
				IAutomation::AddToErrorLog("Loading Data was " + outInfo.ToString(), false);
			}
			else return PREPROCESS::PREPROCESS_SUCCESS;
			return PREPROCESS::PREPROCESS_ERROR;
		}

		static	List<String^>^ GetAllLayerNames()
		{
			//create a list of names foreach layer
			if (mainWindow->Equals(nullptr))
			{
				return nullptr;
			}
			if (!mainWindow->JobIsLoaded) { IAutomation::AddToErrorLog("No data loaded!", true); return nullptr; }

			IStep^ curStep = mainWindow->GetCurrentStep();
			if (curStep->Equals(nullptr)) {
				return nullptr;
			}

			return curStep->GetAllLayerNames(true);
		}

		static	PREPROCESS CreateImage(String^  FullFilePathToSave, String^  LayerName, int DPI)
		{
			//create an image and save it in full file path.
			if (mainWindow->Equals(nullptr)) return PREPROCESS::PREPROCESS_INITERROR;
			if (!mainWindow->JobIsLoaded) { IAutomation::AddToErrorLog("No data loaded!", true); return PREPROCESS::PREPROCESS_NODATA; }

			IStep^ curStep = mainWindow->GetCurrentStep();
			if (curStep->Equals(nullptr)) return PREPROCESS::PREPROCESS_NODATA;

			List<IODBLayer^>^ layerList = gcnew List<IODBLayer^>();

			IODBLayer^ layerODB = (IODBLayer^)curStep->GetLayer(LayerName);
			layerList->Add(layerODB);

			List<bool>^ colors = gcnew List<bool>();
			colors->Add(true);
			System::Drawing::RectangleF bounds = layerODB->GetBounds();
			//for using of step and repeat the bounds are empty
			if (bounds.IsEmpty) bounds = curStep->GetBounds();

			if (curStep->AOIHighResolutionBMPExport(layerList, colors, FullFilePathToSave, bounds, false, false, DPI, AOIMatrixSize::Middle, true, false))
				return PREPROCESS::PREPROCESS_SUCCESS;

			return PREPROCESS::PREPROCESS_CONVERSIONFAIL;
		}
		static	PREPROCESS CreateImage(String^  FullFilePathToSave, String^  LayerName, int DPI, bool Inversion, int MatrixSize, RECTD ImageArea, int matrixSizeIndividual, int ThreadCount)
		{
			//create an image and save it in full file path (many options).
			if (mainWindow->Equals(nullptr)) return PREPROCESS::PREPROCESS_INITERROR;
			if (!mainWindow->JobIsLoaded) { IAutomation::AddToErrorLog("No data loaded!", true); return PREPROCESS::PREPROCESS_NODATA; }

			IStep^ curStep = mainWindow->GetCurrentStep();
			if (curStep->Equals(nullptr)) return PREPROCESS::PREPROCESS_NODATA;

			List<IODBLayer^>^ layerList = gcnew List<IODBLayer^>();

			IODBLayer^ layerODB = (IODBLayer^)curStep->GetLayer(LayerName);
			layerList->Add(layerODB);

			List<bool>^ colors = gcnew List<bool>();
			colors->Add(true);
			//IAutomation::AddToErrorLog("path for image "+ FullFilePathToSave, true);
			AOIMatrixSize size = (AOIMatrixSize)MatrixSize;
			IAutomation::AddToErrorLog("used matrix size enum "+ size.ToString(), true);

			System::Drawing::RectangleF^ imageArea = gcnew System::Drawing::RectangleF();// layerODB->GetBounds();

			imageArea->X = ImageArea.left;
			imageArea->Y = ImageArea.bottom; //maybe change to minimum of top and bottom
			imageArea->Height = Math::Abs(ImageArea.top - ImageArea.bottom); //depending on your rectangle definition the top and bottom is sometimes changed (y direction up or dwon)
			imageArea->Width = ImageArea.right - ImageArea.left;
			//compare area: IAutomation::AddToErrorLog("image area " + imageArea->X+" "+ imageArea->Y +" "+ imageArea->Width+" "+ imageArea->Height, true);

			int notUsed = -1;
			if (curStep->AOIHighResolutionBMPExport(layerList, colors, FullFilePathToSave, *imageArea, false, false, DPI, size, true, Inversion, notUsed, false, false, nullptr))//in beta version you can use his parameter too: , matrixSizeIndividual, ThreadCount))
				return PREPROCESS::PREPROCESS_SUCCESS;

			return PREPROCESS::PREPROCESS_CONVERSIONFAIL;
		}


		static	PREPROCESS CreateImageOfFirstLayer(String^  FullFilePathToSave, int DPI)
		{
			//create a image of first layer in layerlist
			if (mainWindow->Equals(nullptr)) return PREPROCESS::PREPROCESS_INITERROR;
			if (!mainWindow->JobIsLoaded) { IAutomation::AddToErrorLog("No data loaded!", true); return PREPROCESS::PREPROCESS_NODATA; }

			IStep^ curStep = mainWindow->GetCurrentStep();
			if (curStep->Equals(nullptr)) return PREPROCESS::PREPROCESS_NODATA;

			List<String^>^ allLayernames = curStep->GetAllLayerNames(true);
			List<IODBLayer^>^ layerList = gcnew List<IODBLayer^>();

			for (int iCnt = 1; iCnt < allLayernames->Count; iCnt++)
			{
#pragma region take first layer and add is as iodblayer (this works only for gerber data)
				//IODBLayer^ layerODB = (IODBLayer^)curStep->GetLayer(LayerName);

				ILayer^ layer = curStep->GetLayer(allLayernames[iCnt]);
				//if (layer->GetType() == decltype(IODBLayer) )
				{
					layerList->Add((IODBLayer^)layer);
					break;
				}
#pragma endregion
			}

			List<bool>^ colors = gcnew List<bool>();
			colors->Add(true);

			if (curStep->AOIHighResolutionBMPExport(layerList, colors, FullFilePathToSave, curStep->GetBounds(), false, false, DPI, AOIMatrixSize::Middle, true, false))
				return PREPROCESS::PREPROCESS_SUCCESS;

			return PREPROCESS::PREPROCESS_CONVERSIONFAIL;
		}

		static RECTD GetBounds(String^ LayerName)
		{
			//get layer bounds for relevant layername
			RECTD	rect = {};
			if (mainWindow == nullptr) return rect;

			IStep^ step = mainWindow->GetCurrentStep();
			if (step == nullptr) return rect;

			ILayer^ layer = step->GetLayer(LayerName);
			if (layer == nullptr) return rect;

			PCBI::MathUtils::RectangleD^ bounds = layer->GetBoundsD();
			//write to log example IAutomation::AddToErrorLog("bounds "+bounds->X+" "+bounds->Y, true);

			rect.left = bounds->Left;
			rect.right = bounds->Right;
			rect.top = Math::Max(bounds->Top, bounds->Bottom);
			rect.bottom = Math::Min(bounds->Top, bounds->Bottom);
			IAutomation::AddToErrorLog("bounds of Layer " + bounds, false);

			return rect;
		}
		static RECTD GetStepBounds()
		{
			//get a rectD with bounds of step
			RECTD	rect = {};
			if (mainWindow == nullptr) return rect;

			IStep^ step = mainWindow->GetCurrentStep();
			if (step == nullptr) return rect;

			PCBI::MathUtils::RectangleD^ bounds = step->GetBoundsD();

			rect.left = bounds->Left;
			rect.right = bounds->Right;
			rect.top = Math::Max( bounds->Top, bounds->Bottom);
			rect.bottom = Math::Min(bounds->Top, bounds->Bottom);
			IAutomation::AddToErrorLog("bounds of step " + step->GetBoundsD(), false);

			return rect;
		}

		static void RotateData(String^ LayerName, bool Clockwise)
		{
			//rotate a layer with all elements on it.
			if (mainWindow == nullptr) return;

			IStep^ step = mainWindow->GetCurrentStep();
			if (step == nullptr) return;

			ILayer^ layer = step->GetLayer(LayerName);
			if (layer == nullptr) return;

			layer->RotateLayer(Clockwise); //only in 90 degree steps
		}
		static void AddRotatedMirrorStep( double Rotation, bool MirrorX, bool MirrorY)
		{
			//create a new step with rotation, mirrorX and/or mirrorY
			if (mainWindow == nullptr) return;
			
			IFilter^ filter = gcnew IFilter(mainWindow);
			IStep::StepAndRepeatClass^ item = gcnew IStep::StepAndRepeatClass();
			item->ANGLE = Rotation;
			item->MIRRORX = MirrorX;

			if (MirrorY)
			{
				item->ANGLE += 180;
				item->MIRRORX = !item->MIRRORX;
			}
			if (FirstStepName == "") //should not be for gerber!
			{
				IStep^ stepTemp = mainWindow-> GetCurrentStep();
				if(stepTemp == nullptr) return;
				item->NAME = stepTemp->Name;
				IAutomation::AddToErrorLog("No main step is init, please use LoadGerberLayerAndReturnLayerName for adding gerber layers!", false);
			}
			else
				item->NAME = FirstStepName;

			item->NX = 1;
			item->NY = 1;

			IMatrix^ matrix = mainWindow->GetMatrix();
			if (matrix == nullptr)return;
			int count = matrix->GetStepNames(false)->Count;

			String^ newStepName = item->NAME + "_transformed"+ count;

			filter->AddEmptyStep(newStepName);
			IStep^ step = mainWindow->GetStep(newStepName);
			if(step == nullptr) return;
			step->ActivateStep();

			List<IStep::StepAndRepeatClass^>^ newSRs = gcnew List<IStep::StepAndRepeatClass^>();
			newSRs->Add(item);
			step->SetChildSteps(newSRs);
			IAutomation::AddToErrorLog("New step is added: "+newStepName, false);

			//show bounds in log: IAutomation::AddToErrorLog("bounds of step " + step->GetBoundsD(), true);
		}

		static void OversizeingObjects(String^ LayerName, double sizeUpdatePercent)
		{
			//create an oversize on selected layer.
			if (mainWindow == nullptr) return;

			IStep^ step = mainWindow->GetCurrentStep();
			if (step == nullptr) return;

			ILayer^ layer = step->GetLayer(LayerName);
			if (layer == nullptr) return;

			IAutomation::AddToErrorLog("oversize percent value: " + sizeUpdatePercent, false);
			/*if(!((IODBLayer^)layer)->Oversize(sizeUpdatePercent, false, true, 100000))
				IAutomation::AddToErrorLog("error while oversizing...", true);*/
		}

		static void Save()
		{
			//save changes
			if (mainWindow == nullptr) return;
			mainWindow->SaveJobImperative(); //save without questions etc
		}
		static void CloseJob()
		{
			//close the current database.
			if (mainWindow == nullptr) return;
			mainWindow->CloseJob();
		}
		static void DisposeAllResources()
		{
			//dispose the main window to release the resources.
			if (mainWindow == nullptr) return;
			IAutomation::ClosePCBIWindow(mainWindow);
		}
		static void ClearLog()
		{
			//reset all errors (this will also internal called if you open new data)
			IAutomation::ResetErrorLog();
		}
		static void ShowPCBIWindow()
		{
			//Show PCB-Investigator GUI
			if (mainWindow == nullptr) {mainWindow = IAutomation::CreateNewPCBIWindow(false, true); }
			if (mainWindow == nullptr) return;
			IAutomation::SuppressUserNotifications=true; //this is only nesessary if you want to suppress all message boxes
		    //mainWindow->MainForm->ShowDialog();
			mainWindow->Show();
			System::Windows::Forms::Application::Run(mainWindow->MainForm);
		}
		static bool IsPCBIVisible()
		{
			//Check the visibility of the current PCB-Investigator.
			if (mainWindow == nullptr) return false;
			return mainWindow->IsPCBIVisible;
		}

		static	void SelectComponent(String^  CMPRef)
		{
			//search for component reference and select the relevant component
			if (mainWindow->Equals(nullptr)) return;
			if (!mainWindow->JobIsLoaded) { IAutomation::AddToErrorLog("No data loaded!", true); return; }

			IStep^ curStep = mainWindow->GetCurrentStep();
			if (curStep->Equals(nullptr)) return;

			List<ICMPObject^>^ cmpList = curStep->GetAllCMPObjects();

			for each (ICMPObject^ cmp in cmpList)
			{
				if(cmp->Ref == CMPRef)
				{
					cmp->Select(true);
					return;
				}
			}

			IAutomation::AddToErrorLog("Component not found!", false);
		}
		static	void ZoomToComponent(String^  CMPRef)
		{
			//zoom to component bounds
			if (mainWindow->Equals(nullptr)) return;
			if (!mainWindow->JobIsLoaded) { IAutomation::AddToErrorLog("No data loaded!", true); return; }

			IStep^ curStep = mainWindow->GetCurrentStep();
			if (curStep->Equals(nullptr)) return;

			List<ICMPObject^>^ cmpList = curStep->GetAllCMPObjects();

			for each (ICMPObject^ cmp in cmpList)
			{
				if(cmp->Ref == CMPRef)
				{
					mainWindow->ZoomRect( cmp->GetBoundsD()); //maybe it is better to inflate the bounds first, depending on what you want to see.
					return;
				}
			}

			IAutomation::AddToErrorLog("Component not found!", false);
		}
		static	String^ GetComponentInfo(String^  CMPRef)
		{
			//build a string with some component information
			if (mainWindow->Equals(nullptr)) return "";
			if (!mainWindow->JobIsLoaded) { IAutomation::AddToErrorLog("No data loaded!", true); return ""; }

			IStep^ curStep = mainWindow->GetCurrentStep();
			if (curStep->Equals(nullptr)) return "";

			List<ICMPObject^>^ cmpList = curStep->GetAllCMPObjects();

			for each (ICMPObject^ cmp in cmpList)
			{
				if(cmp->Ref == CMPRef)
				{
					//you can add much more information here:
					return cmp->Ref+" Package "+ cmp->UsedPackageName+" Value "+cmp->Value;
				}
			}

			IAutomation::AddToErrorLog("Component not found!", false);
			return "";
		}
		static void ClearAllSelections()
		{
			//clear selection in current step
			if (mainWindow == nullptr) return;
			if (!mainWindow->JobIsLoaded) { IAutomation::AddToErrorLog("No data loaded!", true); return; }

			IStep^ curStep = mainWindow->GetCurrentStep();
			if (curStep->Equals(nullptr)) return;

			curStep->ClearSelection();
		}
		static void ActivateLayersForSelectedObjects()
		{
			//check all selected elements and activate there parent layers.

			if (mainWindow == nullptr) return;
			if (!mainWindow->JobIsLoaded) { IAutomation::AddToErrorLog("No data loaded!", true); return; }

			IStep^ curStep = mainWindow->GetCurrentStep();
			if (curStep->Equals(nullptr)) return;

			//components:
			for each (ICMPObject^ selectedCMP in curStep->GetSelectedCMPs())
			{
				ICMPLayer^ cmpLayer = curStep->GetCMPLayer(selectedCMP->PlacedTop);
				cmpLayer->EnableLayer(true);
			}

			//signal elements:
			for each (IODBObject^ selectedODB in curStep->GetSelectedElements())
			{
				ILayer^ layer =curStep->GetLayer(selectedODB->GetParentLayerName());
				layer->EnableLayer(true);
			}
		}
	
			
#pragma region event example
		static bool RegisterAllEvents(Unmanaged* eventHandlingClass)
		{
			if (mainWindow == nullptr)
			{
				Console::WriteLine("Main Window null! No event handler created!");
				return false;
			}
			Console::WriteLine("add event handling class,");
			PCBI_Lib::PCBIUnmanaged::eventHandlingClassMain =eventHandlingClass; 
			Console::WriteLine("Event registrated: On selection changed");

			mainWindow->PCBISelectionChanged += gcnew System::EventHandler(&OnSelectionChange);
			mainWindow->PCBINewJobOpend += gcnew System::EventHandler(&OnJobOpendChange);
			mainWindow->PCBIActiveLayerChanged += gcnew System::EventHandler(&OnActivateLayer);
			return true;
		}
		static void OnActivateLayer(Object^ sender, System::EventArgs^ updatePacket)
		{
			if (mainWindow == nullptr  ) {
				Console::WriteLine("Main Window null! No event called!");
				return;
			}
			if (eventHandlingClassMain == nullptr  )
			{
				Console::WriteLine("No event handler to call!");
				return;
			}
			eventHandlingClassMain->OnActiveLayerChanged();
		}
		static void OnJobOpendChange(Object^ sender, System::EventArgs^ updatePacket)
		{
			if (mainWindow == nullptr  ) {
				Console::WriteLine("Main Window null! No event called!");
				return;
			}
			if (eventHandlingClassMain == nullptr  )
			{
				Console::WriteLine("No event handler to call!");
				return;
			}
			eventHandlingClassMain->OnDataLoaded();
		}
		static void OnSelectionChange(Object^ sender, System::EventArgs^ updatePacket)
		{
			if (mainWindow == nullptr  ) {
				Console::WriteLine("Main Window null! No event called!");
				return;
			}
			if (eventHandlingClassMain == nullptr  )
			{
				Console::WriteLine("No event handler to call!");
				return;
			}
			eventHandlingClassMain->OnSelectionChanged();
		}  
#pragma endregion

	};
	
}

extern "C"
{
	//Error information of all called methods
	__declspec(dllexport) void __stdcall GetErrorLog(LPSTR retArray, DWORD size)
	{
		String^ log = PCBI_Lib::PCBIUnmanaged::GetErrorLog();
		int max = min(size, log->Length) ;
		for (int i = 0; i < max; i++)
		{
			retArray[i] = log[i];
		}
		retArray[max] = '\0';
	}
	//this is to load e.g. ODB++ data but can also be used for gerber (please use LoadGerberLayerAndReturnLayerName because you need the layername later).
	__declspec(dllexport) int __stdcall LoadData(char* FullPath)
	{
		try
		{
			String ^fullName = gcnew String(FullPath);
			return PCBI_Lib::PCBIUnmanaged::LoadData(fullName);
		}
		catch (char const* e)
		{
			return -1;
		}

	}
	//Main method to add gerber layers, you need the name for later image export
	__declspec(dllexport) int __stdcall LoadGerberLayerAndReturnLayerName(char* FullPath, LPSTR retArray, DWORD size)
	{
		String ^fullName = gcnew String(FullPath);
		String^ layername = PCBI_Lib::PCBIUnmanaged::LoadGerberLayer(fullName);
		if(layername=="") return (int)PCBI_Lib::PREPROCESS::PREPROCESS_ERROR;
		int max = min(size, layername->Length);
		for (int i = 0; i < max; i++)
		{
			retArray[i] = layername[i];
		}
		retArray[max] = '\0';

		return (int)PCBI_Lib::PREPROCESS::PREPROCESS_SUCCESS;
	}
	//Create image with maximum of parameters
	__declspec(dllexport) int  __stdcall CreateImage3(std::string FullFilePathToSave, char * Layername, int DPI, bool Inversion, int MatrixSize, double ImageAreaLeft, double ImageAreaRight, double ImageAreaBottom, double ImageAreaTop, int matrixSizeIndividual, int ThreadCount)
	{
		try
		{
			String ^fullName = gcnew String(FullFilePathToSave.c_str());
			String ^layername = gcnew String(Layername);
			RECTD ImageArea = {};
			ImageArea.left = ImageAreaLeft;
			ImageArea.right = ImageAreaRight;
			ImageArea.bottom = ImageAreaBottom;
			ImageArea.top = ImageAreaTop;

			return PCBI_Lib::PCBIUnmanaged::CreateImage(fullName, layername, DPI, Inversion, MatrixSize, ImageArea, matrixSizeIndividual, ThreadCount);
		}
		catch (char const* e)
		{
			return -1;
		}
	}
	//Create image with more parameters
	__declspec(dllexport) int  __stdcall CreateImage2(std::string FullFilePathToSave, char * Layername, int DPI, bool Inversion, int MatrixSize, double ImageAreaLeft, double ImageAreaRight, double ImageAreaBottom, double ImageAreaTop)
	{
		try
		{
			String ^fullName = gcnew String(FullFilePathToSave.c_str());
			String ^layername = gcnew String(Layername);
			RECTD ImageArea = {};
			ImageArea.left = ImageAreaLeft;
			ImageArea.right = ImageAreaRight;
			ImageArea.bottom = ImageAreaBottom;
			ImageArea.top = ImageAreaTop;

			return PCBI_Lib::PCBIUnmanaged::CreateImage(fullName, layername, DPI, Inversion, MatrixSize, ImageArea, 10000, 9);
		}
		catch (char const* e)
		{
			return -1;
		}
	}
	//Create Image with simple call
	__declspec(dllexport) int __stdcall 	 CreateImage(char*   FullFilePathToSave, char* Layername, int DPI)
	{
		try
		{
			String ^fullName = gcnew String(FullFilePathToSave);
			String ^layername = gcnew String(Layername);
			return PCBI_Lib::PCBIUnmanaged::CreateImage(fullName, layername, DPI);
		}
		catch (char const* e)
		{
			return -1;
		}
	}
	//Get the layerbounds in this step, be carefull this can be empty if you change the step!
	__declspec(dllexport) bool __stdcall 	 GetLayerBounds(char* Layername, RECTD* rect)
	{
		try
		{
			String ^layername = gcnew String(Layername);
			RECTD rect2 = PCBI_Lib::PCBIUnmanaged::GetBounds(layername);

			rect->left = rect2.left;
			rect->right = rect2.right;
			rect->top = rect2.top;
			rect->bottom = rect2.bottom;
		}
		catch (char const* e)
		{
			return false;
		}
		return true;
	}
	//Get the step bounds of current step, this include all layers who loaded!
	__declspec(dllexport) bool __stdcall 	 GetStepBounds(RECTD* rect)
	{
		try
		{
			RECTD rect2 = PCBI_Lib::PCBIUnmanaged::GetStepBounds();

			rect->left = rect2.left;
			rect->right = rect2.right;
			rect->top = rect2.top;
			rect->bottom = rect2.bottom;
		}
		catch (char const* e)
		{
			return false;
		}
		return true;
	}
	//Rotate real data for one layer.
	__declspec(dllexport) void __stdcall 	 RotateData(char* Layername, bool Clockwise)
	{
		String ^layername = gcnew String(Layername);
		PCBI_Lib::PCBIUnmanaged::RotateData(layername, Clockwise);
	}
	//Add a new step with rotation, mirror X/Y to have a fast way to only use the "view" of the needed setting.
	__declspec(dllexport) void __stdcall 	 RotateAndMirrorStep(double Rotation, bool MirrorX, bool MirrorY)
	{
		PCBI_Lib::PCBIUnmanaged::AddRotatedMirrorStep(Rotation, MirrorX, MirrorY);
	}
	//Grow or shrink all objects
	__declspec(dllexport) void __stdcall 	 Oversize(char* Layername,double sizeGrowOrShrinkPercent)
	{
		String ^layername = gcnew String(Layername);
		PCBI_Lib::PCBIUnmanaged::OversizeingObjects(layername, sizeGrowOrShrinkPercent);
	}
	//Save the data to check it in PCB-Investigator
	__declspec(dllexport) void __stdcall 	 Save()
	{
		PCBI_Lib::PCBIUnmanaged::Save();
	}
	//This close all aktive gerberdate e.g. to save ram, but after this you create new ODB structure if you open new gerberfile!
	__declspec(dllexport) void __stdcall 	 CloseJob()
	{
		PCBI_Lib::PCBIUnmanaged::CloseJob();
	}
	//Use this method to dispose all resources of the current PCB-Investigator library.
	__declspec(dllexport) void __stdcall 	 DisposeAllResources()
	{
		PCBI_Lib::PCBIUnmanaged::DisposeAllResources();
	}
	//Remove all entries from log.
	__declspec(dllexport) void __stdcall 	 ClearLog()
	{
		PCBI_Lib::PCBIUnmanaged::ClearLog();
	}
	//Show the window to use PCB-Investigator directly
	__declspec(dllexport) void __stdcall 	 ShowPCBIWindow()
	{
		PCBI_Lib::PCBIUnmanaged::ShowPCBIWindow();
	}
	//Is PCB-Investigator visible?
	__declspec(dllexport) bool __stdcall 	 PCBInvestigatorVisible()
	{
		return PCBI_Lib::PCBIUnmanaged::IsPCBIVisible();
	}
	//Select a component by reference
	__declspec(dllexport) void __stdcall 	 HighlightCMP(char*   CMPReference)
	{
		try
		{
			String ^reference = gcnew String(CMPReference);
			PCBI_Lib::PCBIUnmanaged::SelectComponent(reference);
		}
		catch (char const* e)
		{
		}
	}
	//zoom to component by reference
	__declspec(dllexport) void __stdcall 	 ZoomToCMP(char*   CMPReference)
	{
		try
		{
			String ^reference = gcnew String(CMPReference);
			PCBI_Lib::PCBIUnmanaged::ZoomToComponent(reference);
		}
		catch (char const* e)
		{
		}
	}
	//clear selection in PCB-Invesitgator
	__declspec(dllexport) void __stdcall 	 ClearAllSelections()
	{
		PCBI_Lib::PCBIUnmanaged::ClearAllSelections();
	}
	//check all selected elements and activate layers.
	__declspec(dllexport) void __stdcall 	 ActivateLayersForSelectedObjects()
	{
		PCBI_Lib::PCBIUnmanaged::ActivateLayersForSelectedObjects();
	}
	//Get a sring with component infomrmation
	__declspec(dllexport) void __stdcall GetCMPInfo(char*   CMPReference, LPSTR retArray, DWORD size)
	{
		try
		{
			String ^reference = gcnew String(CMPReference);
		
		String^ retString = PCBI_Lib::PCBIUnmanaged::GetComponentInfo(reference);
		int max = min(size, retString->Length) ;
		for (int i = 0; i < max; i++)
		{
			retArray[i] = retString[i];
		}
		retArray[max] = '\0';
		}
		catch (char const* e)
		{
		}
	}
	//register event
	__declspec(dllexport) bool __stdcall  RegisterEvent(Unmanaged*  eventHandlingClass)
	{
		try
		{
			return PCBI_Lib::PCBIUnmanaged::RegisterAllEvents(eventHandlingClass);
		}
		catch (char const* e)
		{
			return false;
		}
	}
	
}

